@@ -6,7 +6,7 @@ package Sample::Context::Singleton::001::constant;
 
 our $VERSION = v1.0.0;
 
-use Sample::Context::Singleton;
+use Sample::Context::Singleton qw[ contrive ];
 
 contrive 'constant' => (
     value => '42',
